#include <p24FJ128GA204.h>

#include "adc.h"
#include "timer.h"

/* This method configures the 10-bit ADC module with 3.3 V as positive voltage reference, 
 * ground as negative voltage reference and the data result as absolute unsigned value. 
 */
void adc_config(void){
    
// To perform a standard A/D conversion:
//  1. Configure the module:
//      a) Configure port pins as analog inputs by setting the appropriate bits 
//      in the ANSx registers (see Section 11.2 ?Configuring Analog Port Pins 
//      (ANSx)? for more information).

//      b) Select the voltage reference source to match the expected range on 
//      analog inputs (AD1CON2<15:13>).
    AD1CON2bits.PVCFG = 0x0; // Vref+ = AVdd
    AD1CON2bits.NVCFG0 = 0x0; // Vref- = AVss
//      c) Select the positive and negative multiplexer inputs for each channel 
//      (AD1CHS<15:0>).
    AD1CHS = 4; // Channel 4 selected 
//      d) Select the analog conversion clock to match the desired data rate 
//      with the processor clock (AD1CON3<7:0>).
    AD1CON3bits.ADCS = 0xFF; // TAD = 256TCY
//      e) Select the appropriate sample/conversion sequence (AD1CON1<7:4> and
//      AD1CON3<12:8>).
    AD1CON1bits.SSRC = 0x0; // The SAMP bit must be cleared by software to start 
                            // conversion
    AD1CON3bits.SAMC = 0x10; // Auto-sample time of 16 TAD
//      f) For Channel A scanning operations, select the positive channels to be 
//      included (AD1CSSH and AD1CSSL registers).
    // Default config in AD1CSSH and AD1CSSL
//      g) Select how conversion results are presented in the buffer 
//      (AD1CON1<9:8> and AD1CON5 register).
    AD1CON1bits.FORM = 0x0; // Data output Absolute decimal result, unsigned, 
                            // right justified
	AD1CON1bits.MODE12 = 0x0; // Set 10-bit mode
    // Default config in AD1CON5
//      h) Select the interrupt rate (AD1CON2<5:2>).
    AD1CON2bits.SMPI = 0x0; // Interrupts at the completion of the conversion 
                            // for each sample
//      i) Turn on A/D module (AD1CON1<15>).
    AD1CON1bits.ADON = 0x1; // Turn on A/D module
//  2. Configure the A/D interrupt (if required):
//      a) Clear the AD1IF bit (IFS0<13>).
//      b) Enable the AD1IE interrupt (IEC0<13>).
//      c) Select the A/D interrupt priority (IPC3<6:4>).
    // No interrupts for A/D module
    return;
}

/* This method reads the current sampled value by the ADC module. */
unsigned short adc_read(void){

//  3. If the module is configured for manual sampling, set the SAMP bit 
//  (AD1CON1<1>) to begin sampling.
    AD1CON1bits.SAMP = 0x1; // Sampling
    timer2_wait(50); // Sample delay
    AD1CON1bits.SAMP = 0x0; // Holding
    timer2_wait(50); // Hold delay
    
    while(!AD1CON1bits.DONE){}; // Wait until sampling is done
    
	/* Return value in mV */
    return (unsigned short)(((double)(ADC1BUF0*3.3)/1024)*1000);
}