#include "mma7455.h"
#include "uart.h"
#include "spi.h"
#include "stdio.h"

#define MMA7455_CALIBRATION_X_AXIS  0x7F0
#define MMA7455_CALIBRATION_Y_AXIS  0x028
#define MMA7455_CALIBRATION_Z_AXIS  0x7D2

unsigned char int1_risingEdgeDetected;

const char *registersName[MMA7455_NUMREGISTERS]={
    "XOUTL","XOUTH","YOUTL","YOUTH","ZOUTL","ZOUTH",
    "XOUT8","YOUT8","ZOUT8",
    "STATUS",
    "DETSRC",
    "TOUT",
    "(Reserved)",
    "I2CAD",
    "USRINF",
    "WHOAMI",
    "XOFFL","XOFFH","YOFFL","YOFFH","ZOFFL","ZOFFH",
    "MCTL",
    "INTRST",
    "CTL1","CTL2",
    "LDTH",
    "PDTH",
    "PW",
    "LT",
    "TW"
};

/* This method resets the MMA7455 registers. */
void mma7455_reset(void){
    spi1_writeRegister(MMA7455_XOFFL,0x00);
    spi1_writeRegister(MMA7455_XOFFH,0x00);
    spi1_writeRegister(MMA7455_YOFFL,0x00);
    spi1_writeRegister(MMA7455_YOFFH,0x00);
    spi1_writeRegister(MMA7455_ZOFFL,0x00);
    spi1_writeRegister(MMA7455_ZOFFH,0x00);
    spi1_writeRegister(MMA7455_MCTL,0x00);
    spi1_writeRegister(MMA7455_INTRST,0x00);
    spi1_writeRegister(MMA7455_CTL1,0x00);
    spi1_writeRegister(MMA7455_CTL2,0x00);
    spi1_writeRegister(MMA7455_LDTH,0x00);
    spi1_writeRegister(MMA7455_PDTH,0x00);
    spi1_writeRegister(MMA7455_PW,0x00);
    spi1_writeRegister(MMA7455_LT,0x00);
    spi1_writeRegister(MMA7455_TW,0x00);
}

/* This method performs the required initial calibration to obtain proper acceleration measurements. */
void mma7455_calibration(void){
    spi1_writeRegister(MMA7455_XOFFH,((MMA7455_CALIBRATION_X_AXIS & 0xFF00)>>8));
    spi1_writeRegister(MMA7455_XOFFL,(MMA7455_CALIBRATION_X_AXIS & 0xFF));
    
    spi1_writeRegister(MMA7455_YOFFH,((MMA7455_CALIBRATION_Y_AXIS & 0xFF00)>>8));
    spi1_writeRegister(MMA7455_YOFFL,(MMA7455_CALIBRATION_Y_AXIS & 0xFF));
    
    spi1_writeRegister(MMA7455_ZOFFH,((MMA7455_CALIBRATION_Z_AXIS & 0xFF00)>>8));
    spi1_writeRegister(MMA7455_ZOFFL,(MMA7455_CALIBRATION_Z_AXIS & 0xFF));
}

/* This method sets the MMA7455 in measurement mode. */
void mma7455_setupMeasurementMode(void){
    unsigned int rx;
    rx=(spi1_readRegister(MMA7455_MCTL) & ~0x0F) | 0x05;
    spi1_writeRegister(MMA7455_MCTL,rx);
}

/* This method is blocking until a freefall is detected in any axis. */
void mma7455_waitFreefall(void){
    unsigned char freefallDetected = 0;
    char x, y, z;
    printf("..waiting for frefall..\r\n");
    mma7455_setupMeasurementMode();
    while(!freefallDetected){
        x=mma7455_getXAxisAcceleration();
        y=mma7455_getYAxisAcceleration();
        z=mma7455_getZAxisAcceleration();
        if((x<=0x0D || x >= 0xF3) && (y<=0x0D || y >= 0xF3) && (z<=0x0D || z >= 0xF3)){
           freefallDetected = 1;
           printf("FREEFALL!\r\n");
        }
    }
}

/* This method prints the value of all MMA7455 registers. */
void mma7455_printRegisters(void){
    unsigned char i = 0;
    printf("MMA7455 registers readout:\r\n");
    for( i = 1 ; i <= MMA7455_NUMREGISTERS ; i++ ){
        printf("%s : 0x%2x\t", registersName[i-1], spi1_readRegister((unsigned char)(i-1)));
        if( i%6 == 0){
            printf("\r\n");
        }
    }
    printf("\r\n");
}

/* This method prints the current acceleration of all three axis. */
void mma7455_printAxisStatus(void){
    printf("Reading acceleration in X, Y and Z axis\r\n");
    printf("X: %d\tY: %d\tZ: %d\r\n",
            mma7455_getXAxisAcceleration(),
            mma7455_getYAxisAcceleration(),
            mma7455_getZAxisAcceleration());
}

/* This method reads the current acceleration in X axis. */
char mma7455_getXAxisAcceleration(void){
    return (char)spi1_readRegister(MMA7455_XOUT8);
}

/* This method reads the current acceleration in Y axis. */
char mma7455_getYAxisAcceleration(void){
    return (char)spi1_readRegister(MMA7455_YOUT8);
}

/* This method reads the current acceleration in Z axis. */
char mma7455_getZAxisAcceleration(void){
    return (char)spi1_readRegister(MMA7455_ZOUT8);
}
