#include <p24FJ128GA204.h>

#include "system.h"
#include "uart.h"
#include "spi.h"
#include "stdio.h"
#include "adc.h"
#include "mma7455.h"
#include "led.h"
#include "timer.h"

#define SYSTEM_GRIP_STRENGTH_SAMPLES 10
#define SYSTEM_MAX_NUM_OF_DATA_ELEMENTS 100

typedef struct{
    unsigned char typeOfData;
    union{
        unsigned long gripStrengthValue;
        unsigned int timeValue;
    };
}system_dataElement_t;

typedef struct{
    unsigned int numOfElements;
    system_dataElement_t dataElements[SYSTEM_MAX_NUM_OF_DATA_ELEMENTS];
}system_dataStorage_t;

void system_printLogo(void);
void system_pinInit(void);
void system_comInit(void);
void system_adcInit(void);
void system_mma7455Init(void);
unsigned int system_counts2time(unsigned int counts);

unsigned char systemMode;
unsigned char colourCode[6]={
    /*SYSTEM_STANDBY*/              RGB_OFF,
    /*SYSTEM_GRIP_STRENGTH*/        RGB_RED,
    /*SYSTEM_SIT_TO_STAND_TIME*/    RGB_YELLOW,
    /*SYSTEM_STAND_TO_SIT_TIME*/    RGB_BLUE,
    /*SYSTEM_WALK_TIME*/            RGB_GREEN,
    /*SYSTEM_SYNC_DATA*/            RGB_WHITE
};

unsigned char modeSelected;
system_dataStorage_t system_dataStorage;
system_dataElement_t data;
extern const char *modeNames[6];

/* This method wraps all internal initialization calls. */
void system_Init(void){
	/* Configure pins */
    system_pinInit();
	/* Configure communication peripherals */
    system_comInit();
	/* Configure ADC */
    system_adcInit();
	/* Configure accelerometer */
    system_mma7455Init();
	/* Initial system mode is STANDBY*/
    systemMode = SYSTEM_STANDBY;
    modeSelected = 1;
    system_dataStorage.numOfElements=0;
}

/* This method configures the output and input pins of the MCU. */
void system_pinInit(void){
    // Unlock Registers
    asm volatile ("MOV #OSCCON, w1 \n"
                    "MOV #0x46, w2 \n"
                    "MOV #0x57, w3 \n"
                    "MOV.b w2, [w1] \n"
                    "MOV.b w3, [w1] \n"
                    "BCLR OSCCON, #6") ;
    
    // Assign U1TX To Pin RP16 = RC0
    RPOR8bits.RP16R = 3;
    
    // Assign SPI1MISO To Pin RP23 = RC7
    RPINR20bits.SDI1R = 23;
    // Assign SPI1MOSI To Pin RP15 = RB15
    RPOR7bits.RP15R = 7;
    // Assign SPI1SCL To Pin RP0 = RB0
    RPOR0bits.RP0R = 8;
        
    // Lock Registers
    asm volatile ("MOV #OSCCON, w1 \n"
                    "MOV #0x46, w2 \n"
                    "MOV #0x57, w3 \n"
                    "MOV.b w2, [w1] \n"
                    "MOV.b w3, [w1] \n"
                    "BSET OSCCON, #6") ;
    
    // Assign CS To Pin RB9
    ANSBbits.ANSB9 = 0;
    TRISBbits.TRISB9 = 0;
    PORTBbits.RB9 = 0;
    
    // Assign RB2 as analog input
    ANSBbits.ANSB2 = 1 ; // RB2 is analog for AN4
    TRISBbits.TRISB2 = 1 ; // RC0 is +
    
    // Assign RGB LED GPIOs: Red = RB6, Green = RA2, Blue = RC5
    TRISBbits.TRISB6 = 0; 
    LATBbits.LATB6 = 0;
    TRISAbits.TRISA2 = 0;
    LATAbits.LATA2 = 0;
    TRISCbits.TRISC5 = 0;
    LATCbits.LATC5 = 0 ;
}

/* This method initializes the UART, SPI and GPIO peripherals. */
void system_comInit(void){
	/* Configure UART */
    uart1_config();
	/* Print LOGO */
    system_printLogo();
    printf(">> COM INIT\r\n");
    printf("UART1 initialized: 8 bits, No parity, 1 Stop bit, 9600 bps\r\n");
    printf("UART1TX = RC0\r\n");
	/* Configure SPI1 */
    spi1_config(); 
    printf("SPI1 initialized: 16 data bits, 50 kHz\r\n");
    printf("SPI1SCLK = RB0\tSPI1CSn = RB9\r\n");
    printf("SPI1MOSI = RB15\tSPI1MISO = RC7\r\n");
	/* COnfigure GPIO for RGB LED */
    led_config();
    printf("RGB LED initialized: Off\r\n");
    printf("Red = RB6\tGreen = RA2\tBlue = RC5\r\n");
    printf("\r\n");
}

/* This method configures the ADC module. */
void system_adcInit(void){
	/* Configure ADC */
    adc_config();
    printf(">> ADC INIT\r\n");
    printf("AD1 initialized: Channel 4, Vref = 3V3\r\n");
    printf("AD1AN0 = RB2\r\n");
    printf("\r\n");
}

/* This method resets and calibrates the MMA7455 accelerometer sensor. */
void system_mma7455Init(void){
	/* Reset accelerometer registers */
    mma7455_reset();
	/* Calibrate accelerometer */
    mma7455_calibration();
    printf(">> MMA7455 INIT\r\n");
    printf("MMA7455 initialized\r\n");
    mma7455_printRegisters();
    printf("\r\n");
}

/* This method returns the current system mode. */
unsigned char system_getSystemMode(void){
    return systemMode;
}

/* This method sets the next system mode. */
void system_setSystemMode(unsigned char nextSystemMode){
    modeSelected = 0;
    systemMode = nextSystemMode;
    if(systemMode > SYSTEM_SYNC_DATA){
        systemMode = SYSTEM_STANDBY;
    }
    /* Change RGB LED colour */
    led_setColour(colourCode[systemMode]);
}

/* This method stores a new measurement into memory. */
void system_storeDataElement(system_dataElement_t dataElement){
    if(system_dataStorage.numOfElements < SYSTEM_MAX_NUM_OF_DATA_ELEMENTS ){
        system_dataStorage.dataElements[system_dataStorage.numOfElements] = dataElement;
        system_dataStorage.numOfElements++;
        printf("Data element stored.\r\n");
    }else{
        printf("Data element NO stored.\r\n");
    }
}

unsigned long gripStrength[SYSTEM_GRIP_STRENGTH_SAMPLES];

/* This method performs the grip strength measurement. */
void system_gripStrength(void){
    unsigned int i = 0;
    unsigned long max = 0;
    system_dataElement_t data;
	
    while(i<SYSTEM_GRIP_STRENGTH_SAMPLES){
        /* Get ADC sample */
        gripStrength[i] = adc_read();
        /* Check in which section resides the sample and apply the proper conversion */
        if(gripStrength[i] < 713 ){
            gripStrength[i] = 0;
        }else if( 713 <= gripStrength[i] && gripStrength[i] <= 1229 ){
            gripStrength[i] = ((gripStrength[i] - 584)*1000) >> 5;
        }else{
            gripStrength[i] = ((gripStrength[i] - 837)*1000) / 20;
        }
        printf("Measurement %u is %lu grams of force.\r\n",i,gripStrength[i]);
        timer2_wait(500);
        /* Update the maximum grip strength value */
        if(max<gripStrength[i]){
            max = gripStrength[i];
        }     
        i++;   
    }
    
    printf("Maximum grip strength is %lu grams of force.\r\n",max); 
    
	/* Store the measurement */
    data.typeOfData = SYSTEM_GRIP_STRENGTH;
    data.gripStrengthValue = max;
    system_storeDataElement(data);
    
	/* Wait for freefall */
    mma7455_waitFreefall();
}

/* This method performs the sit-to-stand measurement. */
void system_sit2Stand(void){
    unsigned int sit2Stand_ms = 0;
	/* Start counting */
    timer3_start();
	/* Wait for freefall */
    mma7455_waitFreefall();
	/* Convert count value to milliseconds */
    sit2Stand_ms = system_counts2time(timer3_stop());
	/* Store the measurement */
    data.typeOfData = SYSTEM_SIT_TO_STAND_TIME;
    data.timeValue = sit2Stand_ms;
    system_storeDataElement(data);
    printf("Measured Sit-to-Stand time is %u ms\r\n",sit2Stand_ms);
}

/* This method performs the stand-to-sit measurement. */
void system_stand2Sit(void){
    unsigned int stand2Sit_ms = 0;
	/* Start counting */
    timer3_start();
	/* Wait for freefall */
    mma7455_waitFreefall();
	/* Convert count value to milliseconds */
    stand2Sit_ms = system_counts2time(timer3_stop());
	/* Store the measurement */
    data.typeOfData = SYSTEM_STAND_TO_SIT_TIME;
    data.timeValue = stand2Sit_ms;
    system_storeDataElement(data);
    printf("Measured Stand-to-Sit time is %u ms\r\n",stand2Sit_ms);   
}

/* This method performs the walk time measurement. */
void system_walkTime(void){
    unsigned int walk_ms = 0;
	/* Start counting */
    timer3_start();
	/* Wait for freefall */
    mma7455_waitFreefall();
	/* Convert count value to milliseconds */
    walk_ms = system_counts2time(timer3_stop());
	/* Store the measurement */
    data.typeOfData = SYSTEM_WALK_TIME;
    data.timeValue = walk_ms;
    system_storeDataElement(data);
    printf("Measured Walk time is %u ms\r\n",walk_ms);
}

/* This method output all the acquired results over the serial/debug communication link. */
void system_syncData(void){
    unsigned char i = 0; 
	/* Printf all measurements if any */
    if(system_dataStorage.numOfElements != 0 ){
        while(i<system_dataStorage.numOfElements){
            if(system_dataStorage.dataElements[i].typeOfData == SYSTEM_GRIP_STRENGTH){
                printf("Data element %d: %s : %lu grams of force.\r\n",i,
                        modeNames[system_dataStorage.dataElements[i].typeOfData],
                        system_dataStorage.dataElements[i].gripStrengthValue);
            }else{
                printf("Data element %d: %s : %u ms.\r\n",i,
                        modeNames[system_dataStorage.dataElements[i].typeOfData],
                        system_dataStorage.dataElements[i].timeValue);
            }
            i++;
        }
    }else{
        printf("No data elements available.\r\n");
    }
	/* Wait for freefall */
    mma7455_waitFreefall();
}

/* This method transform timer counts to time in milliseconds. */
unsigned int system_counts2time(unsigned int counts){
	/* One period of a 121-Hz counter is 0.00825 seconds, or 8.25 ms.
	 * This is the same as multiplying the count value by 8 and adding 
	 * the count value divided by 4 (0.25) */
    return ((counts * 8) + (counts >> 2));  
}

/* This method prints the logo of the wearable device. */
void system_printLogo(void){
printf(" ________   _______           _        _____    _____      _________   ____  ____  \r\n");
printf("|_   __  | |_   __ \\         / \\      |_   _|  |_   _|    |  _   _  | |_  _||_  _| \r\n");
printf("  | |_ \\_|   | |__) |       / _ \\       | |      | |      |_/ | | \\_|   \\ \\  / /   \r\n");
printf("  |  _|      |  __ /       / ___ \\      | |      | |   _      | |        \\ \\/ /    \r\n");
printf(" _| |_      _| |  \\ \\_   _/ /   \\ \\_   _| |_    _| |__/ |    _| |_       _|  |_    \r\n");
printf("|_____|    |____| |___| |____| |____| |_____|  |________|   |_____|     |______| \r\n");  
printf("\r\n");
printf("       ______     ________   ____   ____   _____     ______   ________       \r\n");            
printf("      |_   _ `.  |_   __  | |_  _| |_  _| |_   _|  .' ___  | |_   __  |         \r\n");         
printf("        | | `. \\   | |_ \\_|   \\ \\   / /     | |   / .'   \\_|   | |_ \\_|          \r\n");        
printf("        | |  | |   |  _| _     \\ \\ / /      | |   | |          |  _| _               \r\n");    
printf("       _| |_.' /  _| |__/ |     \\ ' /      _| |_  \\ `.___.'\\  _| |__/ |         \r\n");         
printf("      |______.'  |________|      \\_/      |_____|  `.____ .' |________|   \r\n");
printf("\r\n");
}